@@ -22,11 +22,10 @@
 
 #include <glib-object.h>
 #include <glib/gprintf.h>
-#if HAVE_GIO
 #include <gio/gio.h>
-#endif
 
 #include "gegl.h"
+#include "gegl/gegl-debug.h"
 #include "gegl-types-internal.h"
 #include "gegl-buffer-types.h"
 #include "gegl-buffer.h"
@@ -36,16 +35,11 @@
 #include "gegl-sampler-nearest.h"
 #include "gegl-sampler-linear.h"
 #include "gegl-sampler-cubic.h"
-#include "gegl-sampler-lanczos.h"
-#include "gegl-sampler-downsharp.h"
-#include "gegl-sampler-downsize.h"
-#include "gegl-sampler-downsmooth.h"
-#include "gegl-sampler-upsharp.h"
-#include "gegl-sampler-upsize.h"
-#include "gegl-sampler-upsmooth.h"
+#include "gegl-sampler-lohalo.h"
 #include "gegl-buffer-index.h"
 #include "gegl-tile-backend.h"
 #include "gegl-buffer-iterator.h"
+#include "gegl-buffer-cl-cache.h"
 
 #if 0
 static inline void
@@ -56,10 +50,10 @@ gegl_buffer_pixel_set (GeglBuffer *buffer,
                        guchar     *buf)
 {
   gint  tile_width  = buffer->tile_storage->tile_width;
-  gint  tile_height  = buffer->tile_storage->tile_width;
+  gint  tile_height = buffer->tile_storage->tile_width;
   gint  px_size     = gegl_buffer_px_size (buffer);
   gint  bpx_size    = babl_format_get_bytes_per_pixel (format);
-  Babl *fish        = NULL;
+  const Babl *fish  = NULL;
 
   gint  abyss_x_total  = buffer->abyss.x + buffer->abyss.width;
   gint  abyss_y_total  = buffer->abyss.y + buffer->abyss.height;
@@ -68,48 +62,46 @@ gegl_buffer_pixel_set (GeglBuffer *buffer,
   gint  buffer_abyss_x = buffer->abyss.x;
   gint  buffer_abyss_y = buffer->abyss.y;
 
-  if (format != buffer->format)
+  if (format != buffer->soft_format)
     {
-      fish = babl_fish (buffer->format, format);
+      fish = babl_fish (buffer->soft_format, format);
     }
 
-  {
-    if (!(buffer_y + y >= buffer_abyss_y &&
-          buffer_y + y < abyss_y_total &&
-          buffer_x + x >= buffer_abyss_x &&
-          buffer_x + x < abyss_x_total))
-      { /* in abyss */
-        return;
-      }
-    else
-      {
-        gint      tiledy = buffer_y + buffer->shift_y + y;
-        gint      tiledx = buffer_x + buffer->shift_x + x;
+  if (!(buffer_y + y >= buffer_abyss_y &&
+        buffer_y + y < abyss_y_total &&
+        buffer_x + x >= buffer_abyss_x &&
+        buffer_x + x < abyss_x_total))
+    { /* in abyss */
+      return;
+    }
+  else
+    {
+      gint      tiledy = buffer_y + buffer->shift_y + y;
+      gint      tiledx = buffer_x + buffer->shift_x + x;
 
-        GeglTile *tile = gegl_tile_source_get_tile ((GeglTileSource *) (buffer),
-                                                gegl_tile_indice (tiledx, tile_width),
-                                                gegl_tile_indice (tiledy, tile_height),
-                                                0);
+      GeglTile *tile = gegl_tile_source_get_tile ((GeglTileSource *) (buffer),
+                                                  gegl_tile_indice (tiledx, tile_width),
+                                                  gegl_tile_indice (tiledy, tile_height),
+                                                  0);
 
-        if (tile)
-          {
-            gint    offsetx = gegl_tile_offset (tiledx, tile_width);
-            gint    offsety = gegl_tile_offset (tiledy, tile_height);
-            guchar *tp;
+      if (tile)
+        {
+          gint    offsetx = gegl_tile_offset (tiledx, tile_width);
+          gint    offsety = gegl_tile_offset (tiledy, tile_height);
+          guchar *tp;
 
-            gegl_tile_lock (tile);
-            tp = gegl_tile_get_data (tile) +
-                 (offsety * tile_width + offsetx) * px_size;
-            if (fish)
-              babl_process (fish, buf, tp, 1);
-            else
-              memcpy (tp, buf, bpx_size);
+          gegl_tile_lock (tile);
+          tp = gegl_tile_get_data (tile) +
+            (offsety * tile_width + offsetx) * px_size;
+          if (fish)
+            babl_process (fish, buf, tp, 1);
+          else
+            memcpy (tp, buf, bpx_size);
 
-            gegl_tile_unlock (tile);
-            gegl_tile_unref (tile);
-          }
-      }
-  }
+          gegl_tile_unlock (tile);
+          gegl_tile_unref (tile);
+        }
+    }
   return;
 }
 #endif
@@ -126,7 +118,6 @@ gegl_buffer_in_abyss( GeglBuffer *buffer,
   gint  abyss_x_total  = buffer_abyss_x + buffer->abyss.width;
   gint  abyss_y_total  = buffer_abyss_y + buffer->abyss.height;
 
-
   gint tiledy = y + buffer_shift_y;
   gint tiledx = x + buffer_shift_x;
 
@@ -143,19 +134,19 @@ gegl_buffer_set_pixel (GeglBuffer *buffer,
                        const Babl *format,
                        gpointer    data)
 {
-  guchar *buf         = data;
-  gint    tile_width  = buffer->tile_storage->tile_width;
-  gint    tile_height = buffer->tile_storage->tile_height;
-  gint    bpx_size    = babl_format_get_bytes_per_pixel (format);
-  Babl   *fish        = NULL;
+  guchar     *buf         = data;
+  gint        tile_width  = buffer->tile_storage->tile_width;
+  gint        tile_height = buffer->tile_storage->tile_height;
+  gint        bpx_size    = babl_format_get_bytes_per_pixel (format);
+  const Babl *fish        = NULL;
 
   gint  buffer_shift_x = buffer->shift_x;
   gint  buffer_shift_y = buffer->shift_y;
-  gint  px_size        = babl_format_get_bytes_per_pixel (buffer->format);
+  gint  px_size        = babl_format_get_bytes_per_pixel (buffer->soft_format);
 
-  if (format != buffer->format)
+  if (format != buffer->soft_format)
     {
-      fish = babl_fish ((gpointer) buffer->format,
+      fish = babl_fish ((gpointer) buffer->soft_format,
                         (gpointer) format);
     }
 
@@ -173,19 +164,15 @@ gegl_buffer_set_pixel (GeglBuffer *buffer,
         gint      indice_y = gegl_tile_indice (tiledy, tile_height);
         GeglTile *tile     = NULL;
 
-        if (buffer->hot_tile &&
-            buffer->hot_tile->x == indice_x &&
-            buffer->hot_tile->y == indice_y)
+        if (buffer->tile_storage->hot_tile &&
+            buffer->tile_storage->hot_tile->x == indice_x &&
+            buffer->tile_storage->hot_tile->y == indice_y)
           {
-            tile = buffer->hot_tile;
+            tile = buffer->tile_storage->hot_tile;
           }
         else
           {
-            if (buffer->hot_tile)
-              {
-                gegl_tile_unref (buffer->hot_tile);
-                buffer->hot_tile = NULL;
-              }
+            _gegl_buffer_drop_hot_tile (buffer);
             tile = gegl_tile_source_get_tile ((GeglTileSource *) (buffer),
                                              indice_x, indice_y,
                                              0);
@@ -207,7 +194,7 @@ gegl_buffer_set_pixel (GeglBuffer *buffer,
               memcpy (tp, buf, bpx_size);
 
             gegl_tile_unlock (tile);
-            buffer->hot_tile = tile;
+            buffer->tile_storage->hot_tile = tile;
           }
       }
   }
@@ -220,19 +207,19 @@ gegl_buffer_get_pixel (GeglBuffer *buffer,
                        const Babl *format,
                        gpointer    data)
 {
-  guchar *buf         = data;
-  gint    tile_width  = buffer->tile_storage->tile_width;
-  gint    tile_height = buffer->tile_storage->tile_height;
-  gint    bpx_size    = babl_format_get_bytes_per_pixel (format);
-  Babl   *fish        = NULL;
+  guchar     *buf         = data;
+  gint        tile_width  = buffer->tile_storage->tile_width;
+  gint        tile_height = buffer->tile_storage->tile_height;
+  gint        bpx_size    = babl_format_get_bytes_per_pixel (format);
+  const Babl *fish        = NULL;
 
   gint  buffer_shift_x = buffer->shift_x;
   gint  buffer_shift_y = buffer->shift_y;
-  gint  px_size        = babl_format_get_bytes_per_pixel (buffer->format);
+  gint  px_size        = babl_format_get_bytes_per_pixel (buffer->soft_format);
 
-  if (format != buffer->format)
+  if (format != buffer->soft_format)
     {
-      fish = babl_fish ((gpointer) buffer->format,
+      fish = babl_fish ((gpointer) buffer->soft_format,
                         (gpointer) format);
     }
 
@@ -251,19 +238,15 @@ gegl_buffer_get_pixel (GeglBuffer *buffer,
         gint      indice_y = gegl_tile_indice (tiledy, tile_height);
         GeglTile *tile     = NULL;
 
-        if (buffer->hot_tile &&
-            buffer->hot_tile->x == indice_x &&
-            buffer->hot_tile->y == indice_y)
+        if (buffer->tile_storage->hot_tile &&
+            buffer->tile_storage->hot_tile->x == indice_x &&
+            buffer->tile_storage->hot_tile->y == indice_y)
           {
-            tile = buffer->hot_tile;
+            tile = buffer->tile_storage->hot_tile;
           }
         else
           {
-            if (buffer->hot_tile)
-              {
-                gegl_tile_unref (buffer->hot_tile);
-                buffer->hot_tile = NULL;
-              }
+            _gegl_buffer_drop_hot_tile (buffer);
             tile = gegl_tile_source_get_tile ((GeglTileSource *) (buffer),
                                            indice_x, indice_y,
                                            0);
@@ -281,7 +264,7 @@ gegl_buffer_get_pixel (GeglBuffer *buffer,
               memcpy (buf, tp, px_size);
 
             /*gegl_tile_unref (tile);*/
-            buffer->hot_tile = tile;
+            buffer->tile_storage->hot_tile = tile;
           }
       }
   }
@@ -293,19 +276,20 @@ gegl_buffer_get_pixel (GeglBuffer *buffer,
 void
 gegl_buffer_flush (GeglBuffer *buffer)
 {
-  g_return_if_fail (GEGL_IS_BUFFER (buffer));
+  GeglTileBackend *backend;
 
-  if (buffer->hot_tile)
+  g_return_if_fail (GEGL_IS_BUFFER (buffer));
+  backend = gegl_buffer_backend (buffer);
+
+  _gegl_buffer_drop_hot_tile (buffer);
+
+  if ((GeglBufferHeader*)(backend->priv->header))
     {
-      gegl_tile_unref (buffer->hot_tile);
-      buffer->hot_tile = NULL;
-    }
-  if ((GeglBufferHeader*)(gegl_buffer_backend (buffer)->header))
-    {
-      ((GeglBufferHeader*)(gegl_buffer_backend (buffer)->header))->x =buffer->extent.x;
-      ((GeglBufferHeader*)(gegl_buffer_backend (buffer)->header))->y =buffer->extent.y;
-      ((GeglBufferHeader*)(gegl_buffer_backend (buffer)->header))->width =buffer->extent.width;
-      ((GeglBufferHeader*)(gegl_buffer_backend (buffer)->header))->height =buffer->extent.height;
+      GeglBufferHeader* header = backend->priv->header;
+      header->x = buffer->extent.x;
+      header->y = buffer->extent.y;
+      header->width =buffer->extent.width;
+      header->height =buffer->extent.height;
     }
 
   gegl_tile_source_command (GEGL_TILE_SOURCE (buffer),
@@ -314,7 +298,7 @@ gegl_buffer_flush (GeglBuffer *buffer)
 
 
 
-static void inline
+static inline void
 gegl_buffer_iterate (GeglBuffer          *buffer,
                      const GeglRectangle *roi, /* or NULL for extent */
                      guchar              *buf,
@@ -325,12 +309,11 @@ gegl_buffer_iterate (GeglBuffer          *buffer,
 {
   gint  tile_width  = buffer->tile_storage->tile_width;
   gint  tile_height = buffer->tile_storage->tile_height;
-  gint  px_size     = babl_format_get_bytes_per_pixel (buffer->format);
+  gint  px_size     = babl_format_get_bytes_per_pixel (buffer->soft_format);
   gint  bpx_size    = babl_format_get_bytes_per_pixel (format);
   gint  tile_stride = px_size * tile_width;
   gint  buf_stride;
   gint  bufy = 0;
-  Babl *fish;
 
   gint  buffer_shift_x = buffer->shift_x;
   gint  buffer_shift_y = buffer->shift_y;
@@ -344,8 +327,8 @@ gegl_buffer_iterate (GeglBuffer          *buffer,
   gint  buffer_abyss_y = buffer->abyss.y + buffer_shift_y;
   gint  abyss_x_total  = buffer_abyss_x + buffer->abyss.width;
   gint  abyss_y_total  = buffer_abyss_y + buffer->abyss.height;
-  gint  i;
-  gint  factor = 1;
+  gint  factor         = 1<<level;
+  const Babl *fish;
 
   /* roi specified, override buffers extent */
   if (roi)
@@ -356,11 +339,6 @@ gegl_buffer_iterate (GeglBuffer          *buffer,
       buffer_y = roi->y + buffer_shift_y;
     }
 
-  for (i = 0; i < level; i++)
-    {
-      factor *= 2;
-    }
-
   buffer_abyss_x /= factor;
   buffer_abyss_y /= factor;
   abyss_x_total  /= factor;
@@ -374,7 +352,7 @@ gegl_buffer_iterate (GeglBuffer          *buffer,
   if (rowstride != GEGL_AUTO_ROWSTRIDE)
     buf_stride = rowstride;
 
-  if (format == buffer->format)
+  if (format == buffer->soft_format)
     {
       fish = NULL;
     }
@@ -383,11 +361,11 @@ gegl_buffer_iterate (GeglBuffer          *buffer,
       if (write)
         {
           fish = babl_fish ((gpointer) format,
-                            (gpointer) buffer->format);
+                            (gpointer) buffer->soft_format);
         }
       else
         {
-          fish = babl_fish ((gpointer) buffer->format,
+          fish = babl_fish ((gpointer) buffer->soft_format,
                             (gpointer) format);
         }
     }
@@ -397,8 +375,6 @@ gegl_buffer_iterate (GeglBuffer          *buffer,
       gint tiledy  = buffer_y + bufy;
       gint offsety = gegl_tile_offset (tiledy, tile_height);
 
-
-
       gint bufx    = 0;
 
       if (!(buffer_y + bufy + (tile_height) >= buffer_abyss_y &&
@@ -586,20 +562,25 @@ void
 gegl_buffer_set_unlocked (GeglBuffer          *buffer,
                           const GeglRectangle *rect,
                           const Babl          *format,
-                          void                *src,
+                          const void          *src,
                           gint                 rowstride)
 {
   if (format == NULL)
-    format = buffer->format;
+    format = buffer->soft_format;
 
+  if (gegl_cl_is_accelerated ())
+    {
+      gegl_buffer_cl_cache_flush (buffer, rect);
+    }
+
+#if 0 /* XXX: not thread safe */
   if (rect && rect->width == 1 && rect->height == 1) /* fast path */
     {
       gegl_buffer_set_pixel (buffer, rect->x, rect->y, format, src);
     }
   else
-    {
-      gegl_buffer_iterate (buffer, rect, src, rowstride, TRUE, format, 0);
-    }
+#endif
+    gegl_buffer_iterate (buffer, rect, (void *) src, rowstride, TRUE, format, 0);
 
   if (gegl_buffer_is_shared(buffer))
     {
@@ -610,8 +591,9 @@ gegl_buffer_set_unlocked (GeglBuffer          *buffer,
 void
 gegl_buffer_set (GeglBuffer          *buffer,
                  const GeglRectangle *rect,
+                 gint                 level,
                  const Babl          *format,
-                 void                *src,
+                 const void          *src,
                  gint                 rowstride)
 {
   g_return_if_fail (GEGL_IS_BUFFER (buffer));
@@ -622,10 +604,7 @@ gegl_buffer_set (GeglBuffer          *buffer,
 }
 
 
-
-
 #if 0
-
 /*
  *  slow nearest neighbour resampler that seems to be
  *  completely correct.
@@ -949,8 +928,10 @@ gegl_buffer_get_unlocked (GeglBuffer          *buffer,
 {
 
   if (format == NULL)
-    format = buffer->format;
+    format = buffer->soft_format;
 
+#if 0
+  /* not thread-safe */
   if (scale == 1.0 &&
       rect &&
       rect->width == 1 &&
@@ -959,12 +940,20 @@ gegl_buffer_get_unlocked (GeglBuffer          *buffer,
       gegl_buffer_get_pixel (buffer, rect->x, rect->y, format, dest_buf);
       return;
     }
+#endif
+
+  if (gegl_cl_is_accelerated ())
+    {
+      gegl_buffer_cl_cache_flush (buffer, rect);
+    }
 
   if (!rect && scale == 1.0)
     {
       gegl_buffer_iterate (buffer, NULL, dest_buf, rowstride, FALSE, format, 0);
       return;
     }
+
+  g_return_if_fail (rect);
   if (rect->width == 0 ||
       rect->height == 0)
     {
@@ -1014,7 +1003,7 @@ gegl_buffer_get_unlocked (GeglBuffer          *buffer,
       sample_buf = g_malloc (buf_width * buf_height * bpp);
       gegl_buffer_iterate (buffer, &sample_rect, sample_buf, GEGL_AUTO_ROWSTRIDE, FALSE, format, level);
 #if 1
-  /* slows testing of rendering code speed to much for now and
+  /* slows testing of rendering code speed too much for now and
    * no time to make a fast implementation
    */
 
@@ -1058,11 +1047,12 @@ gegl_buffer_get_unlocked (GeglBuffer          *buffer,
 
 void
 gegl_buffer_get (GeglBuffer          *buffer,
-                 gdouble              scale,
                  const GeglRectangle *rect,
+                 gdouble              scale,
                  const Babl          *format,
                  gpointer             dest_buf,
-                 gint                 rowstride)
+                 gint                 rowstride,
+                 GeglAbyssPolicy       repeat_mode)
 {
   g_return_if_fail (GEGL_IS_BUFFER (buffer));
   gegl_buffer_get_unlocked (buffer, scale, rect, format, dest_buf, rowstride);
@@ -1076,14 +1066,18 @@ gegl_buffer_get_abyss (GeglBuffer *buffer)
   return &buffer->abyss;
 }
 
+GType
+gegl_sampler_gtype_from_enum (GeglSamplerType sampler_type);
+
 void
 gegl_buffer_sample (GeglBuffer       *buffer,
                     gdouble           x,
                     gdouble           y,
-                    gdouble           scale,
+                    GeglMatrix2      *scale,
                     gpointer          dest,
                     const Babl       *format,
-                    GeglInterpolation interpolation)
+                    GeglSamplerType   sampler_type,
+                    GeglAbyssPolicy    repeat_mode)
 {
   GType desired_type;
   g_return_if_fail (GEGL_IS_BUFFER (buffer));
@@ -1094,11 +1088,21 @@ gegl_buffer_sample (GeglBuffer       *buffer,
   return;
 #endif
 
-  desired_type = gegl_sampler_type_from_interpolation (interpolation);
+  desired_type = gegl_sampler_gtype_from_enum (sampler_type);
 
+  if (!format)
+    format = buffer->soft_format;
+
+  if (format == buffer->soft_format &&
+      sampler_type == GEGL_SAMPLER_NEAREST)
+    {
+      /* XXX: not thread safe */
+      gegl_buffer_get_pixel (buffer, x, y, format, dest);
+      return;
+    }
   /* unset the cached sampler if it dosn't match the needs */
   if (buffer->sampler != NULL &&
-     (!G_TYPE_CHECK_INSTANCE_TYPE (buffer->sampler, desired_type) || 
+     (!G_TYPE_CHECK_INSTANCE_TYPE (buffer->sampler, desired_type) ||
        buffer->sampler_format != format
       ))
     {
@@ -1116,9 +1120,8 @@ gegl_buffer_sample (GeglBuffer       *buffer,
       buffer->sampler_format = format;
       gegl_sampler_prepare (buffer->sampler);
     }
-  gegl_sampler_get (buffer->sampler, x, y, dest);
 
-  /* if (scale < 1.0) do decimation, possibly using pyramid instead */
+  gegl_sampler_get (buffer->sampler, x, y, scale, dest, GEGL_ABYSS_NONE);
 }
 
 void
@@ -1133,15 +1136,13 @@ gegl_buffer_sample_cleanup (GeglBuffer *buffer)
     }
 }
 
-
 void
 gegl_buffer_copy (GeglBuffer          *src,
                   const GeglRectangle *src_rect,
                   GeglBuffer          *dst,
                   const GeglRectangle *dst_rect)
 {
-  Babl         *fish;
-
+  const Babl *fish;
   g_return_if_fail (GEGL_IS_BUFFER (src));
   g_return_if_fail (GEGL_IS_BUFFER (dst));
 
@@ -1155,7 +1156,7 @@ gegl_buffer_copy (GeglBuffer          *src,
       dst_rect = src_rect;
     }
 
-  fish = babl_fish (src->format, dst->format);
+  fish = babl_fish (src->soft_format, dst->soft_format);
 
     {
       GeglRectangle dest_rect_r = *dst_rect;
@@ -1165,8 +1166,10 @@ gegl_buffer_copy (GeglBuffer          *src,
       dest_rect_r.width = src_rect->width;
       dest_rect_r.height = src_rect->height;
 
-      i = gegl_buffer_iterator_new (dst, &dest_rect_r, dst->format, GEGL_BUFFER_WRITE);
-      read = gegl_buffer_iterator_add (i, src, src_rect, src->format, GEGL_BUFFER_READ);
+      i = gegl_buffer_iterator_new (dst, &dest_rect_r, 0, dst->soft_format, 
+                                    GEGL_BUFFER_WRITE, GEGL_ABYSS_NONE);
+      read = gegl_buffer_iterator_add (i, src, src_rect, 0, src->soft_format,
+                                       GEGL_BUFFER_READ, GEGL_ABYSS_NONE);
       while (gegl_buffer_iterator_next (i))
         babl_process (fish, i->data[read], i->data[0], i->length);
     }
@@ -1189,18 +1192,100 @@ gegl_buffer_clear (GeglBuffer          *dst,
       dst_rect->height == 0)
     return;
 
-  pxsize = babl_format_get_bytes_per_pixel (dst->format);
+  pxsize = babl_format_get_bytes_per_pixel (dst->soft_format);
+
+  if (gegl_cl_is_accelerated ())
+    gegl_buffer_cl_cache_invalidate (dst, dst_rect);
 
   /* FIXME: this can be even further optimized by special casing it so
    * that fully voided tiles are dropped.
    */
-  i = gegl_buffer_iterator_new (dst, dst_rect, dst->format, GEGL_BUFFER_WRITE);
+  i = gegl_buffer_iterator_new (dst, dst_rect, 0, dst->soft_format,
+                                GEGL_BUFFER_WRITE, GEGL_ABYSS_NONE);
   while (gegl_buffer_iterator_next (i))
     {
       memset (((guchar*)(i->data[0])), 0, i->length * pxsize);
     }
 }
 
+void            gegl_buffer_set_pattern       (GeglBuffer          *buffer,
+                                               const GeglRectangle *rect,
+                                               GeglBuffer          *pattern,
+                                               gdouble              x_offset,
+                                               gdouble              y_offset)
+{
+  GeglRectangle src_rect = {0,}, dst_rect;
+  int pat_width, pat_height;
+  int cols, rows;
+  int col, row;
+  int width, height;
+
+  pat_width  = gegl_buffer_get_width (pattern);
+  pat_height = gegl_buffer_get_height (pattern);
+  width      = gegl_buffer_get_width (buffer);
+  height     = gegl_buffer_get_height (buffer);
+
+  while (x_offset > pat_width)
+    x_offset -= pat_width;
+  while (y_offset < pat_height)
+    y_offset += pat_height;
+
+  while (x_offset < 0)
+    x_offset += pat_width;
+  while (y_offset > pat_height)
+    y_offset -= pat_height;
+
+  src_rect.width = dst_rect.width = pat_width;
+  src_rect.height = dst_rect.height = pat_height;
+
+  cols = width / pat_width + 1;
+  rows = height / pat_height + 1;
+
+  for (row = 0; row <= rows + 1; row++)
+    for (col = 0; col <= cols + 1; col++)
+      {
+        dst_rect.x = x_offset + (col-1) * pat_width;
+        dst_rect.y = y_offset + (row-1) * pat_height;
+        gegl_buffer_copy (pattern, &src_rect, buffer, &dst_rect);
+      }
+}
+
+void            gegl_buffer_set_color         (GeglBuffer          *dst,
+                                               const GeglRectangle *dst_rect,
+                                               GeglColor           *color)
+{
+  GeglBufferIterator *i;
+  gchar               buf[128];
+  gint                pxsize;
+
+  g_return_if_fail (GEGL_IS_BUFFER (dst));
+  g_return_if_fail (color);
+
+  gegl_color_get_pixel (color, dst->soft_format, buf);
+
+  if (!dst_rect)
+    {
+      dst_rect = gegl_buffer_get_extent (dst);
+    }
+  if (dst_rect->width == 0 ||
+      dst_rect->height == 0)
+    return;
+
+  pxsize = babl_format_get_bytes_per_pixel (dst->soft_format);
+
+  /* FIXME: this can be even further optimized by special casing it so
+   * that fully filled tiles are shared.
+   */
+  i = gegl_buffer_iterator_new (dst, dst_rect, 0, dst->soft_format, 
+                                GEGL_BUFFER_WRITE, GEGL_ABYSS_NONE);
+  while (gegl_buffer_iterator_next (i))
+    {
+      int j;
+      for (j = 0; j < i->length; j++)
+        memcpy (((guchar*)(i->data[0])) + pxsize * j, buf, pxsize);
+    }
+}
+
 GeglBuffer *
 gegl_buffer_dup (GeglBuffer *buffer)
 {
@@ -1208,24 +1293,9 @@ gegl_buffer_dup (GeglBuffer *buffer)
 
   g_return_val_if_fail (GEGL_IS_BUFFER (buffer), NULL);
 
-  new_buffer = gegl_buffer_new (gegl_buffer_get_extent (buffer), buffer->format);
+  new_buffer = gegl_buffer_new (gegl_buffer_get_extent (buffer), buffer->soft_format);
   gegl_buffer_copy (buffer, gegl_buffer_get_extent (buffer),
                     new_buffer, gegl_buffer_get_extent (buffer));
   return new_buffer;
 }
 
-void
-gegl_buffer_sampler (GeglBuffer       *buffer,
-                    gdouble           x,
-                    gdouble           y,
-                    gdouble           scale,
-                    gpointer          dest,
-                    const Babl       *format,
-                    gpointer          sampler2)
-{
-  GeglSampler *sampler = sampler2;
-  g_return_if_fail (GEGL_IS_BUFFER (buffer));
-  g_return_if_fail (GEGL_IS_SAMPLER (sampler));
-
-  gegl_sampler_get (sampler, x, y, dest);
-}
