@@ -60,7 +60,7 @@ typedef struct Priv
   GeglBuffer *in_drawable;
   GeglBuffer *aux_drawable;
   GeglBuffer *out_drawable;
-  Babl       *rgba_float;
+  const Babl *rgba_float;
 
   gint        bx1, by1;
   gint        bx2, by2;    /* mask bounds */
@@ -242,9 +242,9 @@ get_rgba_pixel (void       *data,
       gint i;
       if (!p->in_drawable)
         return;
-      gegl_buffer_sample (p->in_drawable, x, y, 1.0, buf,
+      gegl_buffer_sample (p->in_drawable, x, y, NULL, buf,
                           p->rgba_float,
-                          GEGL_INTERPOLATION_NEAREST);
+                          GEGL_SAMPLER_NEAREST, GEGL_ABYSS_NONE);
       for (i = 0; i < 4; i++)
         pixel[i] = buf[i];
     }
@@ -253,9 +253,9 @@ get_rgba_pixel (void       *data,
       gint i;
       if (!p->aux_drawable)
         return;
-      gegl_buffer_sample (p->aux_drawable, x, y, 1.0, buf,
+      gegl_buffer_sample (p->aux_drawable, x, y, NULL, buf,
                           p->rgba_float,
-                          GEGL_INTERPOLATION_NEAREST);
+                          GEGL_SAMPLER_NEAREST, GEGL_ABYSS_NONE);
       for (i = 0; i < 4; i++)
         pixel[i] = buf[i];
     }
@@ -287,7 +287,7 @@ set_rgba_pixel (void       *data,
     }
 
  gegl_buffer_set (p->out_drawable, &roi,
-                  p->rgba_float, buf,
+                  0, p->rgba_float, buf,
                   GEGL_AUTO_ROWSTRIDE);
 }
 
@@ -937,7 +937,8 @@ process (GeglOperation       *operation,
          GeglBuffer          *input,
          GeglBuffer          *aux,
          GeglBuffer          *output,
-         const GeglRectangle *result)
+         const GeglRectangle *result,
+         gint                 level)
 {
   GeglChantO *o = GEGL_CHANT_PROPERTIES (operation);
 
@@ -976,11 +977,13 @@ gegl_chant_class_init (GeglChantClass *klass)
   operation_class->get_bounding_box = get_bounding_box;
   operation_class->get_required_for_output = get_required_for_output;
 
-  operation_class->name        = "gegl:gluas";
-  operation_class->categories  = "script:misc";
-  operation_class->description =
+  gegl_operation_class_set_keys (operation_class,
+  "name"        , "gegl:gluas",
+  "categories"  , "script:misc",
+  "description" ,
         _("A general purpose filter/composer implementation proxy for the"
-          " lua programming language.");
+          " lua programming language."),
+        NULL);
 }
 
 #endif
